!Use the gold sample SNe Ia of Riess et al.(2004)
!See astro-ph/0402512 
!Version May 2004 (modified by David Rapetti)

!Marginalize anayltically over H_0 with flat prior 
!(equivalent to mariganlize over M, absolute magnitude) 
!resultant log likelihood has arbitary origin, but is returned equal
!to the best-fit value.
module snovae
use cmbtypes
implicit none

 integer, parameter :: SN_num = 157 
 real*8, parameter :: Pi_num = 3.14159265359D0 
 real*8 :: SN_z(SN_num), SN_moduli(SN_num), SN_diagerr(SN_num)
 real*8 :: SN_Ninv(SN_num,SN_Num), SN_Ninvmarge(SN_num,SN_Num)
 real*8 :: aprima, bprima, cprima, a1
 real*8 :: SN_trNinv
 logical :: do_SN_init = .true.
contains


 subroutine SN_init
   character (LEN=1200) :: InLine
   character (LEN=20) :: names(186)
   real*8 :: input (186,3)
   integer gold(157)
   integer i

   if (Feedback > 0) write (*,*) 'reading: supernovae data'
   call OpenTxtFile('data/sn_data_riess.dat',tmp_file_unit)
   read(tmp_file_unit,'(a)') InLine
   do i=1, 186
      read(tmp_file_unit, *) names(i), input(i,:)
   end do
   close(tmp_file_unit)

!Use only those supernovae used for `gold sample'
   gold = (/(I, I=1, 29), 32, 33, 34, 35, 37, (I, I=42,45)&
        ,(I,I=48,57),60,62,63,64,65,68,69,(I,I=71,90),(I,I=92,105)&
        ,107,(I,I=109,121),(I,I=123,131),133,134,135,136,137,138&
        ,(I,I=140,144),(I,I=146,161),163,164,169,170,171,173&
        ,174,(I,I=176,186)/)

   SN_z = input(gold,1)
   SN_moduli = input(gold,2)
   SN_diagerr = 1/input(gold,3)**2 !included the extra velocities (see Riess et al.(2004))
   SN_trNinv = sum(SN_diagerr)
  
!   call OpenTxtFile('data/sn_cov_matrix.dat',tmp_file_unit)
!   do i=1,186
!    do j=1,186
!       read(tmp_file_unit,*) x, y, covmatrix(x,y)
!    end do
!   end do
!   close(tmp_file_unit)
!   SN_Ninv = covmatrix(gold,gold)

!Get matrix after analytic marginalization over M_b
!   tmp = 1
!   tmp = MATMUL(SN_Ninv,tmp)/sqrt(sum(SN_Ninv))
!   do i=1,SN_num
!    do j=1,SN_num
!       SN_Ninvmarge(i,j) = SN_Ninv(i,j) - tmp(i)*tmp(j)
!    end do
!   end do

   do_SN_init = .false.

 end subroutine SN_init

 function SN_LnLike(CMB)
  !Assume this is called just after CAMB with the correct model
  use camb
  type(CMBParams) CMB
  real SN_LnLike
  integer i
  double precision z
  real diffs(SN_num), chisq

  if (do_SN_init) call SN_init

     !$OMP PARALLEL DO DEFAULT(SHARED),SCHEDULE(STATIC), PRIVATE(z,i)
     do i=1, SN_num
        !Obviously this is not v efficient...
        z= SN_z(i)
        diffs(i) = 5*log10((1+z)**2*AngularDiameterDistance(z)) - SN_moduli(i)
     end do
     !$OMP END PARALLEL DO 

     !analytical marginalization over H_0 (equivaltent for M, absolute magnitude) 

     aprima = sum(diffs**2*SN_diagerr)
     bprima = sum(diffs*SN_diagerr)
     cprima = sum(SN_diagerr)
     
     ! to calculate the abslolute chisquare 

     a1=log(cprima/(2*Pi_num))
   
 
     chisq=a1+aprima-((bprima**2)/(cprima))

     if (Feedback > 1) write (*,*) 'SN chisq: ', chisq
     SN_LnLike = chisq/2

 end function SN_LnLike


end module snovae
